﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using HeuristicLab.Common;
using HeuristicLab.DataAccess;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Hive.Contracts.BusinessObjects {

  [StorableClass]
  [DataContract]
  [Serializable]
  public class JobDto : PersistableObject {
    [Storable]
    [DataMember]
    public JobState State { get; set; }
    [Storable]
    [DataMember]
    public Guid UserId { get; set; }
    [Storable]
    [DataMember]
    public SlaveDto Slave { get; set; }
    [Storable]
    [DataMember]
    public Guid? ParentJobId { get; set; }
    [Storable]
    [DataMember]
    public TimeSpan ExecutionTime { get; set; }
    [Storable]
    [DataMember]
    public String Exception { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateCreated { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateCalculated { get; set; }
    [Storable]
    [DataMember]
    public DateTime? DateFinished { get; set; }
    [Storable]
    [DataMember]
    public int Priority { get; set; }
    [Storable]
    [DataMember]
    public int CoresNeeded { get; set; }
    [Storable]
    [DataMember]
    public int MemoryNeeded { get; set; }
    [Storable]
    [DataMember]
    public List<HivePluginInfoDto> PluginsNeeded { get; set; }
    [Storable]
    [DataMember]
    public List<Guid> AssignedResourceIds { get; set; }
    [Storable]
    [DataMember]
    public ProjectDto Project { get; set; }

    public override string ToString() {
      return base.ToString() + "State: " + State + ", [Ressource : " + Slave + " ] , DateCreated: " + DateCreated + ", DateCalculated: " + DateCalculated +
        "Priority: " + Priority + ", CoresNeeded: " + CoresNeeded + "AssignedResources: " + AssignedResourceIds;
    }

    public JobDto() {
      PluginsNeeded = new List<HivePluginInfoDto>();
      AssignedResourceIds = new List<Guid>();
    }
    [StorableConstructor]
    protected JobDto(bool deserializing) : base(deserializing) { }
    protected JobDto(JobDto original, Cloner cloner)
      : base(original, cloner) {
      this.Id = original.Id;
      this.AssignedResourceIds = new List<Guid>(original.AssignedResourceIds);
      this.Slave = cloner.Clone(original.Slave);
      this.CoresNeeded = original.CoresNeeded;
      this.DateCalculated = original.DateCalculated;
      this.DateCreated = original.DateCreated;
      this.DateFinished = original.DateFinished;
      this.Exception = original.Exception;
      this.Id = original.Id;
      this.MemoryNeeded = original.MemoryNeeded;
      this.ParentJobId = original.ParentJobId;
      this.ExecutionTime = original.ExecutionTime;
      this.PluginsNeeded = (original.PluginsNeeded.Select(x => cloner.Clone(x))).ToList();
      this.Priority = original.Priority;
      this.Project = cloner.Clone(original.Project);
      this.State = original.State;
      this.UserId = original.UserId;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JobDto(this, cloner);
    }
  }
}
